@@ -15,6 +15,10 @@ from rich.logging import RichHandler
 from .YamlBlock import YamlFence, CodeFence
 import yaml
 
+from pygments import highlight
+from pygments.lexers import get_lexer_by_name
+from pygments.formatters import HtmlFormatter
+
 
 FORMAT = "%(message)s"
 logging.basicConfig(
@@ -27,13 +31,7 @@ log = logging.getLogger("rich")
 HTML renderer for mistletoe with ROOT code execution and asset injection.
 """
 
-# prismjs = """<script src="{}"></script>\n""".format( "https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/prism.min.js" )
-# prismcss = """<link href="{}" rel="stylesheet" />\n""".format( "https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/themes/prism-tomorrow.min.css" )
-# prismajs = '<script src="https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/plugins/autoloader/prism-autoloader.min.js" integrity="sha512-GP4x8UWxWyh4BMbyJGOGneiTbkrWEF5izsVJByzVLodP8CuJH/n936+yQDMJJrOPUHLgyPbLiGw2rXmdvGdXHA==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>'
-
-
-
-css = """
+core_css = """
 
 html {
         font-family: 'Roboto', sans-serif;
@@ -92,6 +90,13 @@ div.root-block-green pre {
 }
 """
 
+pygments_extra_css = """
+pre {
+  line-height: 1.5em !important;
+  font-size: 1.1em !important;
+}
+"""
+
 
 doct = """
 <!DOCTYPE html>
@@ -117,7 +122,7 @@ doct = """
     <!-- PRISM JS -->
         <!-- PRISM JS : core -->
         <script src="https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/prism.min.js" integrity="sha512-pSVqGtpGygQlhN8ZTHXx1kqkjQr30eM+S6OoSzhHGTjh6DKdfy7WZlo1DNO9bhtM0Imf6xNLznZ7iVC2YUMwJQ==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
-        <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/themes/prism-okaidia.min.css" integrity="sha512-mIs9kKbaw6JZFfSuo+MovjU+Ntggfoj8RwAmJbVXQ5mkAX5LlgETQEweFPI18humSPHymTb5iikEOKWF7I8ncQ==" crossorigin="anonymous" referrerpolicy="no-referrer" />
+        <!-- <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/themes/prism-okaidia.min.css" integrity="sha512-mIs9kKbaw6JZFfSuo+MovjU+Ntggfoj8RwAmJbVXQ5mkAX5LlgETQEweFPI18humSPHymTb5iikEOKWF7I8ncQ==" crossorigin="anonymous" referrerpolicy="no-referrer" /> -->
 
 
         <!-- PRISM JS : autoloader JS -->
@@ -244,6 +249,8 @@ class RootHtmlRenderer(HTMLRenderer, RootExecutor):
             log.error( e )
 
         token.language = "yaml"
+        if self.yaml.get( "hide", False ) == True or self.yaml.get( "hide", False ) == "1" or self.yaml.get( "hide", False ) == 1 :
+            return ""
         return super().render_block_code(token)
 
     def process_yaml( self, yaml ):
@@ -277,124 +284,180 @@ class RootHtmlRenderer(HTMLRenderer, RootExecutor):
 
         return code_block + self.divWrap(imgout, "gnuplot-block", "gnuplot-output-block-%d" % (self.blockid - 1) )
     
+
+    """
+    Code Fence is a custom Code Block token that accepts optional arguments after the language id
+    example:
+    ```cpp in:0
+    ...
+    ```
+
+    the above turns off echoing the input to rendered document
+    """
     def render_code_fence( self, token ):
         log.info( "render_code_fence" )
         return self.render_block_code( token )
-        # return "WOW:" + '  '.join([ "{a}::{b}".format( a=l, b=token.options[l] ) for l in token.options ])
+
+    
+    def optionAsBool( self, options, n, default = False ):
+        # option is false by default
+        if n not in options :
+            return default
+        return  not (options.get( n, "" ).lower() == 'false' or options.get( n, "" ) == '0')
+
+    def process_cpp( self, token ):
+        code_block =  super().render_block_code(token)
+        code = token.children[0].content
+        
+        log.info( "exec: %r" % self.optionAsBool( token.options, "exec", True ) )
+        log.info ("no-exec: %r" % self.no_exec)
+        if self.optionAsBool( token.options, "exec", True ) == False or self.no_exec:
+            log.info( "skipping execution of cpp code block" )
+            return code_block
+
+        # rich.inspect( token.options )
+        # optional class for input code block
+        input_class = token.options.get( ".in", "" ) + " root-block root-block-green"
+        code_block = self.divWrap( code_block, input_class)
+
+        # Execute the codez!
+        output, err, imgs = self.run_cmd( code )
+        output = ("# Block [%d]\n" % self.blockid) + output
+
+        # output controls
+        if self.optionAsBool( token.options, "out", True) == False or self.optionAsBool( token.options, "stdout", True) == False:
+            output = ""
+        elif "out" in token.options or "stdout" in token.options:
+            stdout_filename = token.options.get("out", token.options.get( "stdout", "stdout.dat" ))
+            log.info( "Writing block %d stdout to file: %s" % ( self.blockid, stdout_filename ) )
+            with open( stdout_filename, "w" ) as wf:
+                wf.writelines( output.split() )
+        if self.optionAsBool( token.options, "err", True) == False or self.optionAsBool( token.options, "stderr", True) == False:
+            err = ""
+        elif "err" in token.options or "stderr" in token.options:
+            stderr_filename = token.options.get("err", token.options.get( "stderr", "stderr.dat" ))
+            log.info( "Writing block %d stderr to file: %s" % ( self.blockid, stderr_filename ) )
+            with open( stderr_filename, "w" ) as wf:
+                wf.writelines( err.split() )
+        
+        log.info( "silent? %r" % ('silent' in token.options) )
+        if 'silent' in token.options or 'quiet' in token.options:
+            output = ""
+            err = ""
+
+        if self.optionAsBool( token.options,'in', True) == False:
+            code_block = ""
+        
+        img_class = token.options.get( ".image", "" )
+        if self.optionAsBool( token.options, 'img', True ) == False:
+            imgs = []
+
+        # inject stdoutput 
+        divout = '<div id="{id}" class="root-output" style="text-align: center;">'.format( id="root-output-%d" % (self.blockid) )
+        if len( output + err ):
+            divout += "\n" + divtemplate.format( lang="sh", inner=self.escape_html(output + err) )
+        divout += '</div>'
+
+        # inject images
+        imgout = '<div id="{id}" class="root-images" style="text-align: center;">'.format( id="root-images-%d" % (self.blockid))
+        for i in imgs:
+            imgout += self.process_image_output( i, ucls=img_class )
+        imgout += '</div>'
+
+        lexer = get_lexer_by_name("c++")
+        formatter = HtmlFormatter( style=self.yaml.get( "pygments-theme", "dracula" ))
+        if code_block != ""  and self.yaml.get("highlight", "prismjs").lower() == "pygments" :
+            code_block = highlight(code, lexer, formatter)
+
+        self.blockid = self.blockid + 1
+        return code_block + self.divWrap( divout + imgout, "root-block", "root-output-block-%d" % (self.blockid - 1) )
+
+    def process_js(self, token):
+        code =token.children[0].content
+        template = '<script>\n{content}\n</script>'
+        if "//qin" in code:
+            code_block = "" # dont output the code
+        if "//noexec" in code:
+            return code_block
+        code_block = self.divWrap( code_block, "root-block-green")
+        return code_block + template.format(content=code)
+
+    def process_css(self, token):
+        code =token.children[0].content
+        template = '<style>\n{content}\n</style>'
+        if "/* qin */" in code or "/*qin*/" in code:
+            code_block = "" # dont output the code
+        if "/*noexec*/" in code or "/* noexec */" in code:
+            return code_block
+        code_block = self.divWrap( code_block, "root-block-green")
+        return code_block + template.format(content=code)
+
+    def process_html(self, token):
+        code =token.children[0].content
+        template = '<div>\n{content}\n</div>'
+        if "<!--qin-->" in code or "<!-- qin -->" in code:
+            code_block = "" # dont output the code
+        if "<!--noexec-->" in code or "<!-- noexec -->" in code:
+            return code_block
+        code_block = self.divWrap( code_block, "root-block-green")
+
+        return code_block + template.format(content=code)
 
     def render_block_code(self, token):
         log.info( 'block_code' )
-        code_block =  super().render_block_code(token)
-        code =token.children[0].content
+        
+        # code =token.children[0].content
         # if self.autodraw :
         #     code += autodrawt
         # rich.inspect( token )
-        log.info( code )
-        if token.language:
-            attr = ' class="{}"'.format('language-{}'.format(self.escape_html(token.language)))
-        else:
-            attr = ''
+        # log.info( code )
+        # if token.language:
+        #     attr = ' class="{}"'.format('language-{}'.format(self.escape_html(token.language)))
+        # else:
+        #     attr = ''
 
         if "gnuplot" == token.language or token.options.get( "exec", "" ) == "gnuplot":
             return self.process_gnuplot( token )
+
         if "cpp" == self.escape_html(token.language) or token.options.get( "exec", "" ) == "cpp":
-            
-            if self.no_exec or token.options.get( 'exec', "" ) == 'False' or token.options.get( 'exec', "" ) == '0':
-                return code_block
-
-            if "//noexec" in code:
-                return code_block
-            
-
-            m = re.search( "// {0,1}input:(.+)", code )
-            input_class = ""
-            if m and m.groups()[0]:
-                input_class = m.groups()[0]
-
-            code_block = self.divWrap( code_block, "root-block-green %s" % input_class)
-
-            if self.no_exec:
-                output, err, imgs = ("", "", [""])
-            else :
-                output, err, imgs = self.run_cmd( code )
-            output = ("# Block [%d]\n" % self.blockid) + output
-
-            # CONTROL OPTIONS
-            if "//noout" in code or token.options.get( "out", "" ) == "0":
-                output = ""
-            if "//noerr" in code:
-                err = ""
-            if "//quiet" in code or "//q" in code:
-                output = ""
-                err = ""
-            if "//qin" in code or token.options.get( "in", "" ) == "0":
-                code_block = "" # dont output the code
-            m = re.search( "//.*image:(.+)", code )
-            img_class = ""
-            if m and m.groups()[0]:
-                img_class = m.groups()[0]
-
-            # inject stdoutput 
-            divout = '<div id="{id}" class="root-output" style="text-align: center;">'.format( id="root-output-%d" % (self.blockid) )
-            if len( output + err ):
-                divout += "\n" + divtemplate.format( lang="sh", inner=self.escape_html(output + err) )
-            divout += '</div>'
-
-            # inject images
-            imgout = '<div id="{id}" class="root-images" style="text-align: center;">'.format( id="root-images-%d" % (self.blockid))
-            for i in imgs:
-                imgout += self.process_image_output( i, ucls=img_class )
-            imgout += '</div>'
-
-            if "//qimg" in code or "//!img" in code:
-                imgout = ""
-
-            self.blockid = self.blockid + 1
-            return code_block + self.divWrap( divout + imgout, "root-block", "root-output-block-%d" % (self.blockid - 1) )
+            return self.process_cpp( token )
         
         if "js" == token.language:
-            template = '<script>\n{content}\n</script>'
-            if "//qin" in code:
-                code_block = "" # dont output the code
-            if "//noexec" in code:
-                return code_block
-            code_block = self.divWrap( code_block, "root-block-green")
-            return code_block + template.format(content=code)
-        if "css" == token.language:
-            template = '<style>\n{content}\n</style>'
-            if "/* qin */" in code or "/*qin*/" in code:
-                code_block = "" # dont output the code
-            if "/*noexec*/" in code or "/* noexec */" in code:
-                return code_block
-            code_block = self.divWrap( code_block, "root-block-green")
-            return code_block + template.format(content=code)
-        if "html" == token.language:
-            template = '<div>\n{content}\n</div>'
-            if "<!--qin-->" in code or "<!-- qin -->" in code:
-                code_block = "" # dont output the code
-            if "<!--noexec-->" in code or "<!-- noexec -->" in code:
-                return code_block
-            code_block = self.divWrap( code_block, "root-block-green")
-            return code_block + template.format(content=code)
+            return self.process_js(token)
 
+        if "css" == token.language:
+            return self.process_css(token)
+
+        if "html" == token.language:
+            return self.process_html(token)
+
+        code_block =  super().render_block_code(token)
         return code_block
     
     def loadCSS( self ):
-        mycss = css
+        mycss = ""
         mydir = os.path.dirname(os.path.abspath(__file__))
-        log.info( mydir)
-        prismjs_theme = self.yaml.get("prismjs_theme", "")
-        if  prismjs_theme in [ "a11y-dark","atom-dark","base16-ateliersulphurpool.light","cb","coldark-cold","coldark-dark","coy-without-shadows","darcula","dracula","duotone-dark","duotone-earth","duotone-forest","duotone-light","duotone-sea","duotone-space","ghcolors","gruvbox-dark","gruvbox-light","holi-theme","hopscotch","lucario","material-dark","material-light","material-oceanic","night-owl","nord","one-dark","one-light","pojoaque","shades-of-purple","solarized-dark-atom","synthwave84","vs","vsc-dark-plus","xonokai","z-touch" ]:
-            with open( mydir + "/../data/css/prism-%s.css" % prismjs_theme ) as f:
+        # log.info( mydir)
+        prismjs_theme = self.yaml.get("prismjs_theme", self.yaml.get( "prismjs-theme", "okaidia" ))
+        if  prismjs_theme in [ "a11y-dark","atom-dark","base16-ateliersulphurpool.light","cb","coldark-cold","coldark-dark","coy-without-shadows","darcula","dracula","duotone-dark","duotone-earth","duotone-forest","duotone-light","duotone-sea","duotone-space","ghcolors","gruvbox-dark","gruvbox-light","holi-theme","hopscotch","lucario","material-dark","material-light","material-oceanic","night-owl","nord","one-dark","one-light","pojoaque","shades-of-purple","solarized-dark-atom","synthwave84","vs","vsc-dark-plus","xonokai","z-touch", "coy","dark","funky","okaidia","solarizedlight","tomorrow","twilight" ]:
+            with open( mydir + "/data/css/prismjs/prism-%s.css" % prismjs_theme ) as f:
                 ncss = "".join(f.readlines())
-                log.info( ncss )
+                # log.info( ncss )
                 mycss += "\n" + ncss
         return mycss
 
     def render_document(self, token):
         self.footnotes.update(token.footnotes)
         inner = '\n'.join([self.render(child) for child in token.children])
-        css = self.loadCSS()
+        css = core_css
+        if self.yaml.get("highlight", "prismjs").lower() == "prismjs" :
+            try:
+                css =css + self.loadCSS()
+            except Exception as e:
+                log.error( e )
+        
+        if self.yaml.get("highlight", "prismjs").lower() == "pygments":
+            css = css + HtmlFormatter(style=self.yaml.get( "pygments-theme", "dracula" )).get_style_defs('.highlight') + pygments_extra_css
         return doct.format( 
                 title=self.title, 
                 description=self.description,
