@@ -115,8 +115,12 @@ if args.format == "obsidian":
         log.info( "Making asset directory: %s" % ASSET_DIR )
         os.mkdir( ASSET_DIR )
 
-
-
+#input working dir
+log.info( "I am %s" % __file__ )
+inwdir = os.path.dirname(os.path.abspath(args.input))
+log.info( "input working directory: %s" % inwdir )
+if ASSET_DIR == "":
+    ASSET_DIR = inwdir
 
 
 class RootMd :
